import { v4 as uuid } from 'uuid';
import { clamp, round } from './utils/helpers';
import { EasemateConfig } from './utils/Config';
class DefaultDriver {
    #timeSource = performance.now.bind(performance);
    #callbacks = new Map();
    #durations = new Map();
    #starts = new Map();
    #states = new Map();
    #rafId = null;
    #lastFrameTime = 0;
    constructor(timeSource) {
        if (timeSource) {
            this.#timeSource = timeSource;
        }
    }
    getActiveAnimationIds() {
        return Array.from(this.#states.keys());
    }
    isPaused(id) {
        const state = this.#states.get(id);
        return state?.paused || false;
    }
    play = (targets, keyframes, options) => {
        const id = uuid();
        const baseDuration = keyframes.reduce((sum, k) => sum + (k.duration || EasemateConfig.get('defaultDuration')), 0);
        const rate = EasemateConfig.get('rate') || 1;
        const totalDuration = baseDuration / rate;
        const safeDuration = totalDuration > 0 ? totalDuration : 1;
        const delay = options.delay || 0;
        const startTime = this.#timeSource() + delay;
        this.#durations.set(id, safeDuration);
        this.#starts.set(id, startTime);
        this.#states.set(id, { targets, paused: false, lastProgress: 0, started: false });
        this.#callbacks.set(id, ({ progress, isTick }) => {
            const state = this.#states.get(id);
            if (!state) {
                return;
            }
            let initialTick = false;
            if (!state.started) {
                initialTick = true;
                options.onStart?.(state.targets);
                options.onUpdate?.(0, state.targets);
                state.started = true;
                state.lastProgress = 0;
            }
            if (initialTick && progress === 0) {
                if (safeDuration <= 1 && delay <= 0) {
                    options.onComplete?.(state.targets);
                    this.cancel(id);
                }
                return;
            }
            options.onUpdate?.(progress, state.targets);
            if (progress >= 1 && isTick) {
                options.onComplete?.(state.targets);
                requestAnimationFrame(() => this.cancel(id));
            }
            else if (progress >= 1 && !isTick) {
                options.onComplete?.(state.targets);
            }
        });
        if (!this.#rafId) {
            this.#lastFrameTime = this.#timeSource();
            this.#rafId = requestAnimationFrame(this.#tick);
        }
        if (EasemateConfig.get('debug')) {
            console.log(`[Easemate Driver] Animation ${id} scheduled. Duration: ${safeDuration.toFixed(2)}ms, Delay: ${delay}ms`, targets);
        }
        return id;
    };
    #tick = (timestamp) => {
        const delta = timestamp - this.#lastFrameTime;
        this.#lastFrameTime = timestamp;
        const maxFps = EasemateConfig.get('maxFps') || 120;
        const frameInterval = 1000 / maxFps;
        if (delta < frameInterval && !EasemateConfig.get('useSubFrames')) {
            this.#rafId = requestAnimationFrame(this.#tick);
            return;
        }
        let activeAnimationsExist = false;
        for (const id of [...this.#callbacks.keys()]) {
            const callback = this.#callbacks.get(id);
            if (!callback) {
                continue;
            }
            const state = this.#states.get(id);
            if (!state) {
                this.#callbacks.delete(id);
                continue;
            }
            if (state.paused) {
                activeAnimationsExist = true;
                continue;
            }
            const start = this.#starts.get(id) || 0;
            if (timestamp < start) {
                activeAnimationsExist = true;
                continue;
            }
            activeAnimationsExist = true;
            const duration = this.#durations.get(id) || 1;
            let progress = clamp((timestamp - start) / duration, 0, 1);
            const precision = EasemateConfig.get('precision');
            if (precision !== null) {
                progress = round(progress, precision);
            }
            if (state.started) {
                if (EasemateConfig.get('smoothing')) {
                    progress = this.#smoothProgress(state.lastProgress, progress);
                }
                if (EasemateConfig.get('useSubFrames')) {
                    progress = this.#predictSubFrame(progress, delta, duration);
                    progress = clamp(progress, 0, 1);
                }
            }
            state.lastProgress = progress;
            EasemateConfig.callPluginHooks('tick', this, state.targets, null, null, progress);
            callback({ progress, isTick: true });
            if (progress < 1) {
                activeAnimationsExist = true;
            }
            else {
                activeAnimationsExist = false;
            }
        }
        if (activeAnimationsExist) {
            this.#rafId = requestAnimationFrame(this.#tick);
        }
        else {
            this.#rafId = null;
            if (EasemateConfig.get('debug') && this.#callbacks.size > 0) {
                console.log('[Easemate Driver] Animation loop stopped (only paused/finished animations remain).');
            }
            else if (EasemateConfig.get('debug')) {
                console.log('[Easemate Driver] Animation loop stopped (no active animations).');
            }
        }
    };
    #predictSubFrame(progress, delta, duration) {
        if (duration <= 0) {
            return progress;
        }
        const frameRate = EasemateConfig.get('maxFps') || 60;
        const timePerFrame = 1000 / frameRate;
        const subFrameFactor = delta / timePerFrame;
        const progressPerFrame = 1 / (duration / timePerFrame);
        return progress + progressPerFrame * (subFrameFactor - 1);
    }
    #smoothProgress(lastProgress, currentProgress) {
        const factor = EasemateConfig.get('smoothingFactor');
        const delta = currentProgress - lastProgress;
        if (typeof factor === 'number') {
            return lastProgress + delta * clamp(factor, 0, 1);
        }
        switch (factor) {
            case 'linear':
                return lastProgress + delta * 0.5;
            case 'exponential':
                return lastProgress + delta * (1 - Math.exp(-5 * Math.abs(delta)));
            case 'cubic':
                return lastProgress + delta * Math.pow(Math.abs(delta), 3);
            default:
                return currentProgress;
        }
    }
    pause(id) {
        const state = this.#states.get(id);
        if (state && !state.paused) {
            state.paused = true;
            if (EasemateConfig.get('debug')) {
                console.log(`[Easemate Driver] Animation ${id} paused at progress ${state.lastProgress.toFixed(3)}.`);
            }
        }
    }
    resume(id) {
        const state = this.#states.get(id);
        const duration = this.#durations.get(id);
        if (state && state.paused && duration !== undefined) {
            state.paused = false;
            this.#starts.set(id, this.#timeSource() - state.lastProgress * duration);
            if (!this.#rafId) {
                this.#lastFrameTime = this.#timeSource();
                this.#rafId = requestAnimationFrame(this.#tick);
            }
            if (EasemateConfig.get('debug')) {
                console.log(`[Easemate Driver] Animation ${id} resumed from progress ${state.lastProgress.toFixed(3)}.`);
            }
        }
    }
    cancel(id) {
        const existed = this.#callbacks.has(id);
        this.#callbacks.delete(id);
        this.#durations.delete(id);
        this.#starts.delete(id);
        this.#states.delete(id);
        if (existed) {
            if (EasemateConfig.get('debug')) {
                console.log(`[Easemate Driver] Animation ${id} cancelled.`);
            }
        }
        if (this.#callbacks.size === 0 && this.#rafId !== null) {
            if (typeof cancelAnimationFrame !== 'undefined') {
                cancelAnimationFrame(this.#rafId);
            }
            this.#rafId = null;
            if (EasemateConfig.get('debug') && existed) {
                console.log('[Easemate Driver] Animation loop stopped (last animation cancelled).');
            }
        }
    }
    setProgress(id, progress) {
        const callback = this.#callbacks.get(id);
        const state = this.#states.get(id);
        const duration = this.#durations.get(id);
        if (callback && state && duration !== undefined) {
            let adjustedProgress = progress;
            const precision = EasemateConfig.get('precision');
            if (precision !== null) {
                adjustedProgress = round(adjustedProgress, precision);
            }
            adjustedProgress = clamp(adjustedProgress, 0, 1);
            state.lastProgress = adjustedProgress;
            this.#starts.set(id, this.#timeSource() - adjustedProgress * duration);
            callback({ progress: adjustedProgress, isTick: false });
            if (EasemateConfig.get('debug')) {
                console.log(`[Easemate Driver] Animation ${id} progress set to ${adjustedProgress.toFixed(3)}.`);
            }
            if (!this.#rafId && adjustedProgress < 1 && !state.paused) {
                this.#lastFrameTime = this.#timeSource();
                this.#rafId = requestAnimationFrame(this.#tick);
            }
        }
    }
    getState(id) {
        const state = this.#states.get(id);
        if (!state) {
            return undefined;
        }
        const startTime = this.#starts.get(id) || 0;
        const duration = this.#durations.get(id) || 1;
        const currentTime = this.#timeSource();
        let progress;
        if (state.paused) {
            progress = state.lastProgress;
        }
        else if (!state.started || currentTime < startTime) {
            progress = 0;
        }
        else {
            progress = clamp((currentTime - startTime) / duration, 0, 1);
        }
        const precision = EasemateConfig.get('precision');
        if (precision !== null) {
            progress = round(progress, precision);
        }
        return { progress, targets: state.targets, startTime };
    }
}
export class Animate {
    #driver;
    #drivers = new Map();
    #animations = new Map();
    constructor(timeSource) {
        this.#driver = new DefaultDriver(timeSource);
        this.#drivers.set('default', this.#driver);
    }
    registerDriver(type, driver) {
        this.#drivers.set(type, driver);
    }
    setDriver(typeOrDriver) {
        if (typeof typeOrDriver === 'string') {
            const driver = this.#drivers.get(typeOrDriver);
            if (driver) {
                this.#driver = driver;
            }
            else {
                console.warn(`[Easemate Animate] Driver type "${typeOrDriver}" not registered. Using current driver.`);
            }
        }
        else {
            this.#driver = typeOrDriver;
        }
        return this;
    }
    add(targets, keyframes, options = {}) {
        const id = this.#driver.play(targets, keyframes, options);
        this.#animations.set(id, { targets, keyframes });
        return id;
    }
    pause(id) {
        this.#driver.pause(id);
        return this;
    }
    resume(id) {
        this.#driver.resume(id);
        return this;
    }
    cancel(id) {
        this.#driver.cancel(id);
        this.#animations.delete(id);
        return this;
    }
    setProgress(id, progress) {
        this.#driver.setProgress(id, progress);
        return this;
    }
    getTargets(id) {
        return this.#animations.get(id)?.targets;
    }
    getState(id) {
        if (this.#driver.getState) {
            return this.#driver.getState(id);
        }
        return undefined;
    }
    getActiveAnimationIds() {
        return this.#driver.getActiveAnimationIds();
    }
    isPaused(id) {
        return this.#driver.isPaused(id);
    }
}
//# sourceMappingURL=Animate.js.map