import { Animate } from './Animate';
import { Motion } from './Motion';
import { Ease } from './Ease';
import { Flow } from './Flow';
import { Plugin } from './Plugin';
import { Modulate } from './Modulate';
import { EasemateConfig, PluginHookRegistry } from './utils/Config';
export class Easemate {
    static #_animate;
    static plugins = new Map();
    static customEasings = new Map();
    static pausedAnimations = new Set();
    static #visibilityListenerAttached = false;
    static get animate() {
        if (!this.#_animate) {
            this.#_animate = new Animate();
        }
        return this.#_animate;
    }
    static mapEasing(easing) {
        if (this.customEasings.has(easing)) {
            const easingFn = this.customEasings.get(easing);
            if (easingFn) {
                return easingFn;
            }
        }
        switch (easing) {
            case 'linear':
                return (t) => t;
            case 'ease':
                return (t) => (t < 0.5 ? 2 * t * t : -1 + (4 - 2 * t) * t);
            case 'easeIn':
                return (t) => t * t;
            case 'easeOut':
                return (t) => t * (2 - t);
            case 'easeInOut':
                return (t) => (t < 0.5 ? 2 * t * t : -1 + (4 - 2 * t) * t);
            default:
                return (t) => t;
        }
    }
    static registerEasing(name, fn) {
        this.customEasings.set(name, fn);
    }
    static mate(...plugins) {
        const validated = Plugin.validatePluginChain(plugins);
        validated.forEach((plugin) => {
            this.plugins.set(plugin.id, plugin);
            PluginHookRegistry.register({
                ...plugin.getHooks(),
                ...plugin.getInterpolate(),
            });
            const onRegister = plugin.getHooks().onRegister;
            if (onRegister) {
                onRegister.call(this);
            }
        });
    }
    static flow(config) {
        return new Flow(config, Easemate.animate);
    }
    static init(config) {
        EasemateConfig.reset(config);
        if (EasemateConfig.get('debug')) {
            console.info('Easemate initialized with config:', EasemateConfig.getAll());
        }
        if (typeof window !== 'undefined' && EasemateConfig.get('pauseOnHidden') && !this.#visibilityListenerAttached) {
            this.#visibilityListenerAttached = true;
            window.addEventListener('visibilitychange', () => {
                const isHidden = document.visibilityState === 'hidden';
                if (!this.#_animate) {
                    return;
                }
                const allActiveIds = this.animate.getActiveAnimationIds();
                if (isHidden) {
                    allActiveIds.forEach((id) => {
                        if (!this.animate.isPaused(id)) {
                            const state = this.animate.getState(id);
                            if (state && state.progress < 1) {
                                this.animate.pause(id);
                                this.pausedAnimations.add(id);
                            }
                        }
                    });
                }
                else {
                    [...this.pausedAnimations].forEach((id) => {
                        if (this.animate.getState(id)) {
                            this.animate.resume(id);
                        }
                        this.pausedAnimations.delete(id);
                    });
                }
            });
        }
    }
    static reset() {
        this.#_animate = undefined;
        this.plugins.clear();
        this.customEasings.clear();
        this.pausedAnimations.clear();
        this.#visibilityListenerAttached = false;
        PluginHookRegistry.clear();
        EasemateConfig.reset();
    }
}
export const ease = (target, config) => {
    return new Ease(target, config, Easemate.animate);
};
export const flow = Easemate.flow;
export const mate = Easemate.mate;
export { Motion, Animate, Ease, Plugin, Modulate, EasemateConfig };
//# sourceMappingURL=index.js.map