"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Ease = void 0;
const Motion_1 = require("./Motion");
const index_1 = require("./index");
const Modulate_1 = require("./Modulate");
const helpers_1 = require("./utils/helpers");
const Config_1 = require("./utils/Config");
class Ease extends Motion_1.Motion {
    #targets;
    #to;
    #config;
    #current;
    #Modulate;
    getInitialTargets() {
        return this.#targets;
    }
    constructor(targets, config = {}, animate) {
        super(animate);
        this.#targets = ((0, helpers_1.isArray)(targets) ? targets : [targets]);
        const fallback = (_index, val) => val;
        this.#to = config.to ?? fallback;
        const easingDefault = config.easing === undefined && config.delay ? 'linear' : Config_1.EasemateConfig.get('defaultEase');
        this.#config = {
            duration: Config_1.EasemateConfig.get('defaultDuration'),
            easing: index_1.Easemate.mapEasing(easingDefault),
            delay: 0,
            repeat: 0,
            autoplay: Config_1.EasemateConfig.get('autoplay'),
            direction: 'forward',
            ...config,
        };
        if (typeof this.#config.duration === 'number' && this.#config.duration < 0) {
            throw new Error('Duration cannot be negative');
        }
        Config_1.EasemateConfig.callPluginHooks('beforeInit', this, this.#targets, this.#to, this.#config);
        this.#current = [...this.#targets];
        this.#Modulate = new Modulate_1.Modulate();
        Config_1.EasemateConfig.callPluginHooks('init', this, this.#targets, this.#to, this.#config);
        if (Config_1.EasemateConfig.get('debug')) {
            console.log('Ease initialized with targets:', this.#targets, 'config:', this.#config);
        }
        if (this.#config.autoplay) {
            this.play(this.#config.duration || Config_1.EasemateConfig.get('defaultDuration'), this.#config.delay || 0);
        }
        Config_1.EasemateConfig.callPluginHooks('afterInit', this, this.#targets, this.#to, this.#config);
    }
    update(progress) {
        const easing = this.#config.easing ?? index_1.Easemate.mapEasing('linear');
        let eased = progress;
        if (this.#config.repeat && this.#config.repeat > 0) {
            const totalCycles = this.#config.repeat + 1;
            const fullProgress = progress * totalCycles;
            const cycleProgress = fullProgress % 1;
            eased = fullProgress >= totalCycles ? 1 : cycleProgress;
        }
        eased = typeof easing === 'string' ? index_1.Easemate.mapEasing(easing)(eased) : easing(eased);
        if (this.#config.direction === 'reverse') {
            eased = 1 - eased;
        }
        this.#current = this.#targets.map((from, i) => {
            const to = this.#resolveTo(from, i);
            return this.#Modulate.interpolate(from, to, eased);
        });
        if (this.#config.onUpdate) {
            this.#current.forEach((target, i) => this.#config.onUpdate?.(target, this.#resolveTo(target, i), progress, this.#current));
        }
        this.#config.onProgress?.(progress, this.#current);
        Config_1.EasemateConfig.callPluginHooks('afterUpdate', this, this.#current, this.#resolveToProps(), this.#config);
        if (progress >= 1) {
            Config_1.EasemateConfig.callPluginHooks('beforeComplete', this, this.#current, this.#to, this.#config);
            if (this.#config.onComplete) {
                this.#current.forEach((target, i) => this.#config.onComplete?.(target, this.#resolveTo(target, i), this.#current));
            }
            Config_1.EasemateConfig.callPluginHooks('afterComplete', this, this.#current, this.#to, this.#config);
        }
        return this.#current;
    }
    play(duration = this.#config.duration ?? Config_1.EasemateConfig.get('defaultDuration'), delay = this.#config.delay ?? 0, keyframes) {
        if (this.id) {
            this.getAnimate().cancel(this.id);
        }
        const endValue = this.getInterpolatedValue(1)[0];
        this.id = this.getAnimate().add(this.#current, keyframes || [{ value: endValue, duration }], {
            delay: delay,
            onStart: (targets) => {
                if (this.#config.onStart) {
                    this.#targets.forEach((_initialTarget, i) => {
                        const currentTarget = targets[i];
                        const resolvedProps = this.#resolveTo(this.#targets[i], i);
                        this.#config.onStart?.(currentTarget, resolvedProps, targets);
                    });
                }
                Config_1.EasemateConfig.callPluginHooks('start', this, targets, null, { duration, delay });
            },
            onUpdate: (progress, _targets) => {
                const updated = this.update(progress);
                Config_1.EasemateConfig.callPluginHooks('update', this, updated, null, { duration, delay });
            },
            onComplete: (targets) => {
                Config_1.EasemateConfig.callPluginHooks('complete', this, targets, null, { duration, delay });
            },
        });
        if (Config_1.EasemateConfig.get('debug')) {
            console.log(`Ease ${this.id} playing with duration: ${duration}ms, delay: ${delay}ms`);
        }
        return this;
    }
    getInterpolatedValue(progress) {
        const easing = this.#config.easing ?? index_1.Easemate.mapEasing('linear');
        let eased = progress;
        if (this.#config.repeat && this.#config.repeat > 0) {
            const totalCycles = this.#config.repeat + 1;
            const fullProgress = progress * totalCycles;
            const cycleProgress = fullProgress % 1;
            eased = fullProgress >= totalCycles ? 1 : cycleProgress;
        }
        eased = typeof easing === 'string' ? index_1.Easemate.mapEasing(easing)(eased) : easing(eased);
        if (this.#config.direction === 'reverse') {
            eased = 1 - eased;
        }
        return this.#targets.map((from, i) => {
            const to = this.#resolveTo(from, i);
            return this.#Modulate.interpolate(from, to, eased);
        });
    }
    #resolveTo = (from, index) => {
        if ((0, helpers_1.isFunction)(this.#to)) {
            return this.#to(index, from, this.#targets);
        }
        if (Array.isArray(this.#to)) {
            return this.#to[index];
        }
        return this.#to;
    };
    #resolveToProps() {
        if ((0, helpers_1.isFunction)(this.#to)) {
            return this.#to(0, this.#targets[0], this.#targets);
        }
        return this.#to;
    }
    getCurrent() {
        return this.#current;
    }
    getProgress() {
        const id = this.getId();
        return id ? (this.getAnimate().getState?.(id)?.progress ?? 0) : 0;
    }
}
exports.Ease = Ease;
//# sourceMappingURL=Ease.js.map