"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Plugin = void 0;
const Config_1 = require("./utils/Config");
const helpers_1 = require("./utils/helpers");
class Plugin {
    id;
    #hooks;
    #interpolateConfig;
    #enabled = true;
    #priority = 0;
    #dependencies = [];
    constructor(id, hooks = {}, interpolateConfig = {}) {
        this.id = id;
        this.#hooks = hooks;
        this.#interpolateConfig = interpolateConfig;
    }
    get enabled() {
        return this.#enabled;
    }
    enable() {
        this.#enabled = true;
        return this;
    }
    disable() {
        this.#enabled = false;
        return this;
    }
    get priority() {
        return this.#priority;
    }
    setPriority(priority) {
        this.#priority = (0, helpers_1.clamp)(priority, 0, 100);
        if (Config_1.EasemateConfig.get('debug')) {
            console.log(`Plugin ${this.id} priority set to ${this.#priority}`);
        }
        return this;
    }
    get dependencies() {
        return this.#dependencies.slice();
    }
    setDependencies(...pluginIds) {
        this.#dependencies = pluginIds;
        return this;
    }
    addDependency(pluginId) {
        if (!this.#dependencies.includes(pluginId)) {
            this.#dependencies.push(pluginId);
        }
        return this;
    }
    removeDependency(pluginId) {
        this.#dependencies = this.#dependencies.filter((id) => id !== pluginId);
        return this;
    }
    hasDependency(pluginId) {
        return this.#dependencies.includes(pluginId);
    }
    getHooks() {
        return this.#hooks;
    }
    getInterpolate() {
        return this.#interpolateConfig;
    }
    validateDependencies(registeredPlugins) {
        const registeredIds = registeredPlugins.map((p) => p.id);
        const missing = this.#dependencies.filter((dep) => !registeredIds.includes(dep));
        if (missing.length && Config_1.EasemateConfig.get('debug')) {
            console.warn(`Plugin ${this.id} missing dependencies:`, missing);
        }
        return this.#dependencies.every((dep) => registeredIds.includes(dep));
    }
    static validatePluginChain(plugins) {
        const sorted = [...plugins].sort((a, b) => b.priority - a.priority);
        const usedIds = new Set();
        for (const plugin of sorted) {
            if (usedIds.has(plugin.id)) {
                throw new Error(`Duplicate plugin ID: ${plugin.id}`);
            }
            usedIds.add(plugin.id);
            if (!plugin.validateDependencies(sorted)) {
                throw new Error(`Plugin "${plugin.id}" has unmet dependencies`);
            }
        }
        return sorted;
    }
}
exports.Plugin = Plugin;
//# sourceMappingURL=Plugin.js.map