"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EasemateConfig = exports.Modulate = exports.Plugin = exports.Ease = exports.Animate = exports.Motion = exports.mate = exports.flow = exports.ease = exports.Easemate = void 0;
const Animate_1 = require("./Animate");
Object.defineProperty(exports, "Animate", { enumerable: true, get: function () { return Animate_1.Animate; } });
const Motion_1 = require("./Motion");
Object.defineProperty(exports, "Motion", { enumerable: true, get: function () { return Motion_1.Motion; } });
const Ease_1 = require("./Ease");
Object.defineProperty(exports, "Ease", { enumerable: true, get: function () { return Ease_1.Ease; } });
const Flow_1 = require("./Flow");
const Plugin_1 = require("./Plugin");
Object.defineProperty(exports, "Plugin", { enumerable: true, get: function () { return Plugin_1.Plugin; } });
const Modulate_1 = require("./Modulate");
Object.defineProperty(exports, "Modulate", { enumerable: true, get: function () { return Modulate_1.Modulate; } });
const Config_1 = require("./utils/Config");
Object.defineProperty(exports, "EasemateConfig", { enumerable: true, get: function () { return Config_1.EasemateConfig; } });
class Easemate {
    static #_animate;
    static plugins = new Map();
    static customEasings = new Map();
    static pausedAnimations = new Set();
    static #visibilityListenerAttached = false;
    static get animate() {
        if (!this.#_animate) {
            this.#_animate = new Animate_1.Animate();
        }
        return this.#_animate;
    }
    static mapEasing(easing) {
        if (this.customEasings.has(easing)) {
            const easingFn = this.customEasings.get(easing);
            if (easingFn) {
                return easingFn;
            }
        }
        switch (easing) {
            case 'linear':
                return (t) => t;
            case 'ease':
                return (t) => (t < 0.5 ? 2 * t * t : -1 + (4 - 2 * t) * t);
            case 'easeIn':
                return (t) => t * t;
            case 'easeOut':
                return (t) => t * (2 - t);
            case 'easeInOut':
                return (t) => (t < 0.5 ? 2 * t * t : -1 + (4 - 2 * t) * t);
            default:
                return (t) => t;
        }
    }
    static registerEasing(name, fn) {
        this.customEasings.set(name, fn);
    }
    static mate(...plugins) {
        const validated = Plugin_1.Plugin.validatePluginChain(plugins);
        validated.forEach((plugin) => {
            this.plugins.set(plugin.id, plugin);
            Config_1.PluginHookRegistry.register({
                ...plugin.getHooks(),
                ...plugin.getInterpolate(),
            });
            const onRegister = plugin.getHooks().onRegister;
            if (onRegister) {
                onRegister.call(this);
            }
        });
    }
    static flow(config) {
        return new Flow_1.Flow(config, Easemate.animate);
    }
    static init(config) {
        Config_1.EasemateConfig.reset(config);
        if (Config_1.EasemateConfig.get('debug')) {
            console.info('Easemate initialized with config:', Config_1.EasemateConfig.getAll());
        }
        if (typeof window !== 'undefined' && Config_1.EasemateConfig.get('pauseOnHidden') && !this.#visibilityListenerAttached) {
            this.#visibilityListenerAttached = true;
            window.addEventListener('visibilitychange', () => {
                const isHidden = document.visibilityState === 'hidden';
                if (!this.#_animate) {
                    return;
                }
                const allActiveIds = this.animate.getActiveAnimationIds();
                if (isHidden) {
                    allActiveIds.forEach((id) => {
                        if (!this.animate.isPaused(id)) {
                            const state = this.animate.getState(id);
                            if (state && state.progress < 1) {
                                this.animate.pause(id);
                                this.pausedAnimations.add(id);
                            }
                        }
                    });
                }
                else {
                    [...this.pausedAnimations].forEach((id) => {
                        if (this.animate.getState(id)) {
                            this.animate.resume(id);
                        }
                        this.pausedAnimations.delete(id);
                    });
                }
            });
        }
    }
    static reset() {
        this.#_animate = undefined;
        this.plugins.clear();
        this.customEasings.clear();
        this.pausedAnimations.clear();
        this.#visibilityListenerAttached = false;
        Config_1.PluginHookRegistry.clear();
        Config_1.EasemateConfig.reset();
    }
}
exports.Easemate = Easemate;
const ease = (target, config) => {
    return new Ease_1.Ease(target, config, Easemate.animate);
};
exports.ease = ease;
exports.flow = Easemate.flow;
exports.mate = Easemate.mate;
//# sourceMappingURL=index.js.map